DROP FUNCTION alert_check_memory(bigint, bigint);
CREATE OR REPLACE FUNCTION alert_check_memory(mem1 bigint, mem2 bigint) RETURNS integer AS $$
    SELECT
        CASE WHEN round($1::float / 65536) != round($2::float / 65536) THEN 1 ELSE 0 END;
$$
LANGUAGE SQL;

DROP FUNCTION alert_brazil(machine_type_enum, date);
CREATE OR REPLACE FUNCTION alert_brazil(machtype machine_type_enum, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("hd" bigint, "memory" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		SUM(
			CASE WHEN
				i1.disk1_model != i2.disk1_model
				OR
				i1.disk1_size != i2.disk1_size
				OR
				(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
				OR
				(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))
			THEN 1
			ELSE 0 END
		),
		SUM(alert_check_memory(i1.memory, i2.memory))
	FROM
		dim_inventory i1
	INNER JOIN
		dim_inventory i2
	ON
		i1.tl_id = i2.tl_id AND i1.macaddr = i2.macaddr AND
		i1.is_first = '1' AND i2.is_current = '1'
	WHERE
		i1.machine_type = machtype AND
		i2.load_date <= b_date;
END;
$$ language plpgsql;

DROP FUNCTION alert_brazil_monthly_history(machine_type_enum);
CREATE OR REPLACE FUNCTION alert_brazil_monthly_history(machtype machine_type_enum) RETURNS
	TABLE (data text, "hd" bigint, "memory" bigint) AS $$
BEGIN
	RETURN QUERY
	WITH dates AS (
		SELECT (CURRENT_DATE - (i || ' month'):: interval)::date AS dat
		FROM
			generate_series(5, 0, -1) i
	)
	SELECT
		get_month_name(extract(month from dat)::integer)::text as c,
		(alert_brazil).*
	FROM
		(SELECT 
			alert_brazil(machtype, dat),
			dat
		FROM
			dates
		) AS a;
END;
$$ language plpgsql;

DROP FUNCTION alert_region(machine_type_enum, date);
CREATE OR REPLACE FUNCTION alert_region(machtype machine_type_enum, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("region" varchar, "hd" bigint, "memory" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		c.region,
		SUM(
			CASE WHEN
				i1.disk1_model != i2.disk1_model
				OR
				i1.disk1_size != i2.disk1_size
				OR
				(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
				OR
				(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))
			THEN 1
			ELSE 0 END
		),
		SUM(alert_check_memory(i1.memory, i2.memory))
	FROM
		dim_inventory i1
	INNER JOIN
		dim_inventory i2
	ON
		i1.tl_id = i2.tl_id AND i1.macaddr = i2.macaddr AND
		i1.is_first = '1' AND i2.is_current = '1'
	INNER JOIN
		dim_telecenter t
	ON
		i1.tl_id = t.id
	INNER JOIN
		dim_city c
	ON
		t.cit_id = c.id
	WHERE
		i1.machine_type = machtype AND
		i2.load_date <= b_date
	GROUP BY
		c.region;

END;
$$ language plpgsql;

DROP FUNCTION alert_region_monthly_history(machine_type_enum, text);
CREATE OR REPLACE FUNCTION alert_region_monthly_history(machtype machine_type_enum, region_in text) RETURNS
	TABLE (data text, "hd" bigint, "memory" bigint)
AS $$
	SELECT inn.c, sum(inn.hd)::bigint, sum(inn.memory)::bigint FROM
		(SELECT
			get_month_name(extract(month from dat)::integer)::text as c,
			(alert_state).*
		FROM
			(SELECT 
				alert_state($1, $2, dat),
				dat
			FROM
				(SELECT (CURRENT_DATE - (i || ' month'):: interval)::date AS dat
				FROM
					generate_series(5, 0, -1) i
				) AS dates
			) AS a
		) AS inn
	GROUP BY
		inn.c
	ORDER BY
		inn.c DESC;
$$
LANGUAGE SQL;

DROP FUNCTION alert_state(machine_type_enum, text, date);
CREATE OR REPLACE FUNCTION alert_state(machtype machine_type_enum, region_in text, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("state" varchar, "hd" bigint, "memory" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		c.state,
		SUM(
			CASE WHEN
				i1.disk1_model != i2.disk1_model
				OR
				i1.disk1_size != i2.disk1_size
				OR
				(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
				OR
				(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))
			THEN 1
			ELSE 0 END
		),
		SUM(alert_check_memory(i1.memory, i2.memory))
	FROM
		dim_inventory i1
	INNER JOIN
		dim_inventory i2
	ON
		i1.tl_id = i2.tl_id AND i1.macaddr = i2.macaddr AND
		i1.is_first = '1' AND i2.is_current = '1'
	INNER JOIN
		dim_telecenter t
	ON
		i1.tl_id = t.id
	INNER JOIN
		dim_city c
	ON
		t.cit_id = c.id
	WHERE
		i1.machine_type = machtype AND
		c.region = region_in AND
		i2.load_date <= b_date
	GROUP BY
		c.region, c.state;

END;
$$ language plpgsql;

DROP FUNCTION alert_state_monthly_history(machine_type_enum, text);
CREATE OR REPLACE FUNCTION alert_state_monthly_history(machtype machine_type_enum, state_in text) RETURNS
	TABLE (data text, "hd" bigint, "memory" bigint)
AS $$
	SELECT inn.c, sum(inn.hd)::bigint, sum(inn.memory)::bigint FROM
		(SELECT
			get_month_name(extract(month from dat)::integer)::text as c,
			(alert_city).*
		FROM
			(SELECT 
				alert_city($1, $2, dat),
				dat
			FROM
				(SELECT (CURRENT_DATE - (i || ' month'):: interval)::date AS dat
				FROM
					generate_series(5, 0, -1) i
				) AS dates
			) AS a
		) AS inn
	GROUP BY
		inn.c
	ORDER BY
		inn.c DESC;
$$
LANGUAGE SQL;

DROP FUNCTION alert_city(machine_type_enum, text, date);
CREATE OR REPLACE FUNCTION alert_city(machtype machine_type_enum, state_in text, b_date DATE DEFAULT CURRENT_DATE) RETURNS
	TABLE ("city" varchar, "hd" bigint, "memory" bigint) AS $$
BEGIN
	RETURN QUERY SELECT
		c.name,
		SUM(
			CASE WHEN
				i1.disk1_model != i2.disk1_model
				OR
				i1.disk1_size != i2.disk1_size
				OR
				(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
				OR
				(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))
			THEN 1
			ELSE 0 END
		),
		SUM(alert_check_memory(i1.memory, i2.memory))
	FROM
		dim_inventory i1
	INNER JOIN
		dim_inventory i2
	ON
		i1.tl_id = i2.tl_id AND i1.macaddr = i2.macaddr AND
		i1.is_first = '1' AND i2.is_current = '1'
	INNER JOIN
		dim_telecenter t
	ON
		i1.tl_id = t.id
	INNER JOIN
		dim_city c
	ON
		t.cit_id = c.id
	WHERE
		i1.machine_type = machtype AND
		c.state = state_in AND
		i2.load_date <= b_date
	GROUP BY
		c.name;

END;
$$ language plpgsql;

DROP FUNCTION alert_city_monthly_history(machine_type_enum, text);
CREATE OR REPLACE FUNCTION alert_city_monthly_history(machtype machine_type_enum, city_in text) RETURNS
	TABLE ("hd" bigint, "memory" bigint, data text) AS $$
BEGIN
	RETURN QUERY
	WITH dates AS (
		SELECT (CURRENT_DATE - (i || ' month'):: interval)::date AS dat
		FROM
			generate_series(5, 0, -1) i
	)
	SELECT sum(al.memory), sum(al.hd), get_month_name(extract(month from dat)::integer)::text as c
	FROM
		(SELECT 
			alert_city(machtype, city_in, dat) as al,
			dat
		FROM
			dates
		) AS a
	GROUP BY dat
	ORDER BY dat;
END;
$$ language plpgsql;

CREATE OR REPLACE FUNCTION alert_report(machtype machine_type_enum, region_in text, state_in text, city_in text) RETURNS
	TABLE ("load_date" text, "region" text, "state" varchar, "city" text, "telecenter" text, "machine" macaddr, "old_date" date, "new_date" date, "os_distro" text, "processor" text, "old_memory_size" integer, "new_memory_size" integer, "old_hd_size" integer, "new_hd_size" integer, "memory_alert" integer, "hd_alert" integer) AS $$
BEGIN
	RETURN QUERY SELECT
		to_char((SELECT max(base_date) FROM aggr_availability), 'DD/MM/YYYY'),
		initcap(c.region),
		c.state,
		initcap(c.name),
		initcap(t.name),
		i1.macaddr,
		i1.load_date, i2.load_date,
		i1.os_distro, i1.processor,
		i1.memory, i2.memory,
		i1.disk1_size, i2.disk1_size,
		alert_check_memory(i1.memory, i2.memory),
		CASE WHEN
			i1.disk1_model != i2.disk1_model
			OR
			i1.disk1_size != i2.disk1_size
			OR
			(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
			OR
			(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))
		THEN 1 ELSE 0 END
	FROM
		dim_inventory i1
	INNER JOIN
		dim_inventory i2
	ON
		i1.tl_id = i2.tl_id AND i1.macaddr = i2.macaddr AND
		i1.is_first = '1' AND i2.is_current = '1'
	INNER JOIN
		dim_telecenter t
	ON
		i1.tl_id = t.id
	INNER JOIN
		dim_city c
	ON
		t.cit_id = c.id
	WHERE
		i1.machine_type = machtype AND c.region = region_in AND c.state = state_in AND c.name = city_in
		AND
		((alert_check_memory(i1.memory, i2.memory) = 1) OR (i1.disk1_model != i2.disk1_model OR i1.disk1_size != i2.disk1_size OR
			(i1.disk2_model != i2.disk2_model AND (i1.disk2_model IS NOT NULL OR i2.disk2_model IS NOT NULL))
			OR
			(i1.disk2_size != i2.disk2_size AND (i1.disk2_size IS NOT NULL OR i2.disk2_size IS NOT NULL))))
	ORDER BY
		t.name, i1.macaddr, i1.load_date;
END;
$$ language plpgsql;
